"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _isPlainObject2 = require("lodash/isPlainObject");

var _isPlainObject3 = _interopRequireDefault(_isPlainObject2);

var _forEach2 = require("lodash/forEach");

var _forEach3 = _interopRequireDefault(_forEach2);

exports.getAllModulesFromFile = getAllModulesFromFile;

var _luaparse = require("luaparse");

var lua = _interopRequireWildcard(_luaparse);

var _fs = require("fs");

var fs = _interopRequireWildcard(_fs);

var _path = require("path");

var path = _interopRequireWildcard(_path);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function getAllModulesFromFile(filePath) {
    var fileASTs = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    filePath = path.resolve(filePath);

    if (fileASTs[filePath]) return fileASTs;

    var folderPath = path.dirname(filePath);
    var code = fs.readFileSync(filePath, "utf-8");
    var ast = lua.parse(code);

    var container = fileASTs[filePath] = {
        submodules: {},
        ast: ast
    };

    (function traverse(node) {
        var submodule = getSubmodule(node);
        if (submodule) {
            getAllModulesFromFile(submodule, fileASTs);
        }

        (0, _forEach3.default)(node, function (child) {
            if (Array.isArray(child)) (0, _forEach3.default)(child, function (subnode) {
                return traverse(subnode);
            });else if ((0, _isPlainObject3.default)(child)) traverse(child);
        });
    })(ast);

    return fileASTs;

    // --------------------------------------------------------

    function getSubmodule(node) {
        if (!node || !node.type) return;

        if (node.type !== "StringCallExpression" && node.type !== "CallExpression") return;

        if (node.base.name !== "require") return;

        if ((node.argument || node.arguments[0]).type !== "StringLiteral") return;

        var submodule = (node.argument || node.arguments[0]).value;
        submodule = path.extname(submodule).length > 0 ? submodule : submodule + ".lua";
        submodule = path.resolve(folderPath, submodule);

        container.submodules[submodule] = node;
        return submodule;
    }
}
//# sourceMappingURL=moduleDependencies.js.map