"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _forEach2 = require("lodash/forEach");

var _forEach3 = _interopRequireDefault(_forEach2);

exports.shortenedModuleNames = shortenedModuleNames;
exports.changeModuleRequireToLoader = changeModuleRequireToLoader;
exports.numberToAlphabet = numberToAlphabet;
exports.createLuaModuleLoaderAST = createLuaModuleLoaderAST;
exports.replaceAST = replaceAST;

var _luaparse = require("luaparse");

var lua = _interopRequireWildcard(_luaparse);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function shortenedModuleNames(modulesData) {
    var output = {};
    var oldPaths = {};
    var index = 0;

    (0, _forEach3.default)(modulesData, function (container, path) {
        var id = numberToAlphabet(index);
        output[id] = container;
        oldPaths[path] = id;
        index++;
    });

    (0, _forEach3.default)(modulesData, function (container) {
        var shortenedSubmodules = {};

        (0, _forEach3.default)(container.submodules, function (ast, submodulePath) {
            shortenedSubmodules[oldPaths[submodulePath]] = ast;
        });

        container.submodules = shortenedSubmodules;
    });

    changeModuleRequireToLoader(output);
    return output;
}

function changeModuleRequireToLoader(modulesData) {
    (0, _forEach3.default)(modulesData, function (container, moduleName) {
        (0, _forEach3.default)(container.submodules, function (requireAST, submoduleName) {
            replaceAST(requireAST, createLuaModuleLoaderAST(submoduleName).expression);
        });
    });
}

function numberToAlphabet(n) {
    var posA = "a".charCodeAt(0);
    var posB = "z".charCodeAt(0);
    var length = posB - posA + 1;
    var output = "";

    while (n >= 0) {
        output = String.fromCharCode(n % length + posA) + output;
        n = Math.floor(n / length) - 1;
    }

    return output;
}

function createLuaModuleLoaderAST(moduleName) {
    return lua.parse("\n        __lc_module(\"" + moduleName + "\")\n    ").body[0];
}

function replaceAST(ast, newAST) {
    (0, _forEach3.default)(ast, function (value, key) {
        delete ast[key];
    });

    (0, _forEach3.default)(newAST, function (value, key) {
        ast[key] = value;
    });

    return ast;
}
//# sourceMappingURL=utils.js.map